using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class ORB : Indicator
    {
    
    	double high, low;
    	Trend trend = Trend.Nutral;
    
        public ORB() : this(new TimeSpan(10, 0, 0))
        { }

        public ORB(TimeSpan period) : base()
        {   
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                
                	AddPlot(new Plot("Up", new HashPlotStyle(), Brushes.Green, 2));
                	AddPlot(new Plot("Down", new HashPlotStyle(), Brushes.Red, 2));
                
                	this.high = double.MinValue;
                	this.low = double.MaxValue;
                
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
        	if (!this.Bars.BarType.IsIntraday)
        		throw new Exception("ORB indicator works on intraday bars only");
        
            if (this.Bars.GetBar(CurrentBar).IsFirstBarOfSession)
            {
            	this.high = High[0];
            	this.low = Low[0];
            	
            	this.trend = Trend.Nutral;
            	
            	Values[0][0] = this.high;
            	Values[1][0] = this.low;
            	
            	return;
            }
            
            if (Time[0].TimeOfDay <= this.Period)
            {
            	Values[0][0] = this.high = Math.Max(this.high, High[0]);
            	Values[1][0] = this.low = Math.Min(this.low, Low[0]);
            	
            	this.trend = Trend.Nutral;
            }
            else
            {
            	Values[0][0] = this.high;
            	Values[1][0] = this.low;
            	
            	if (this.State != State.Realtime)
            	{
            		
           			if (Close[0] < this.low)
           			{
           				this.trend = Trend.Down;
           			}
           			else if (Close[0] > this.high)
           			{
           				this.trend = Trend.Up;
           			}
           			
           			return;
            	}	
            
            	switch (this.trend)
            	{
            		case Trend.Up:
            		
            			if (Close[0] < this.low)
            			{
            				this.trend = Trend.Down;
            				this.Alert("Down" + CurrentBar.ToString(), string.Format("{0} broke opening range low of {1}", this.Bars.Instrument.DisplayName, this.low));
            			}
            		
            			break;
            		case Trend.Down:
            		
            			if (Close[0] > this.high)
            			{
            				this.trend = Trend.Up;
            				this.Alert("Up" + CurrentBar.ToString(), string.Format("{0} broke opening range high of {1}", this.Bars.Instrument.DisplayName, this.high));
            			}
            			break;
            		default:
            		
            			if (Close[0] < this.low)
            			{
            				this.trend = Trend.Down;
            				this.Alert("Down" + CurrentBar.ToString(), string.Format("{0} broke opening range low of {1}", this.Bars.Instrument.DisplayName, this.low));
            			}
            			else if (Close[0] > this.high)
            			{
            				this.trend = Trend.Up;
            				this.Alert("Up" + CurrentBar.ToString(), string.Format("{0} broke opening range high of {1}", this.Bars.Instrument.DisplayName, this.high));
            			}
            			
            			break;
            	}
            }
            
            
        }

        private TimeSpan period = new TimeSpan(10, 0, 0);
        [Input]
        [XmlIgnore]
        public TimeSpan Period
        {
            get { return period; }
            set 
            {
                period = value;
                NotifyPropertyChanged("Period");
            }
        }
		
		public long TimeHelper
		{
			get { return this.Period.Ticks; }
			set
			{
				this.Period = new TimeSpan(value);
			}
			
		}		
		
    }
}